@@ -1,9 +1,9 @@
 import uuid
 
 from django.db.models import Sum
+from django.template.defaultfilters import date as date_filter
 from django.utils.translation import ugettext_lazy as _
 
-from .decorators import report_field_register
 from .helpers import get_calculation_annotation
 from .registry import field_registry
 
@@ -110,6 +110,21 @@ class SlickReportField(object):
             queryset = queryset.aggregate(annotation)
         return queryset
 
+    def init_preparation(self, q_filters=None, kwargs_filters=None, **kwargs):
+        """
+        Called by the generator to preparet he calculation of this field + it's requirements
+        :param q_filters:
+        :param kwargs_filters:
+        :param kwargs:
+        :return:
+        """
+        kwargs_filters = kwargs_filters or {}
+
+        dep_values = self._prepare_dependencies(q_filters, kwargs_filters.copy())
+
+        debit_results, credit_results = self.prepare(q_filters, kwargs_filters, **kwargs)
+        self._cache = debit_results, credit_results, dep_values
+
     def prepare(self, q_filters=None, kwargs_filters=None, **kwargs):
         """
         This is the first hook where you can customize the calculation away from the Django Query aggregation method
@@ -122,10 +137,6 @@ class SlickReportField(object):
         :param kwargs:
         :return:
         """
-        kwargs_filters = kwargs_filters or {}
-
-        dep_values = self._prepare_dependencies(q_filters, kwargs_filters.copy())
-
         queryset = self.get_queryset()
         if q_filters:
             queryset = queryset.filter(*q_filters)
@@ -148,8 +159,7 @@ class SlickReportField(object):
 
             credit_results = self.apply_aggregation(queryset, self.group_by)
 
-        self._cache = debit_results, credit_results, dep_values
-        return debit_results, credit_results, dep_values
+        return debit_results, credit_results
 
     def get_queryset(self):
         queryset = self.report_model.objects
@@ -167,15 +177,16 @@ class SlickReportField(object):
         for dep_class in self._require_classes:
             dep = dep_class(self.plus_side_q, self.minus_side_q, self.report_model,
                             date_field=self.date_field, group_by=self.group_by)
-            values[dep.name] = {'results': dep.prepare(q_filters, extra_filters),
+            values[dep.name] = {'results': dep.init_preparation(q_filters, extra_filters),
                                 'instance': dep}
         return values
 
-    def resolve(self, current_obj):
+    def resolve(self, current_obj, current_row=None):
         '''
         Reponsible for getting the exact data from the prepared value
         :param cached: the returned data from prepare
-        :param current_obj:
+        :param current_obj: he value of group by id
+        :param current_row: the row in iteration
         :return: a solid number or value
         '''
         cached = self._cache
@@ -284,15 +295,28 @@ class SlickReportField(object):
         return f'{cls.verbose_name} {model} {id}'
 
     @classmethod
-    def get_time_series_field_verbose_name(cls, date_period):
+    def get_time_series_field_verbose_name(cls, date_period, index, dates, pattern):
         """
-        Sent the column data to construct a verbose name.
-        Default implemenetation is column name + the end date %Y%m%d
-        :param column_name: the computation field_name
+        Get the name of the verbose name of a computaion field that's in a time_series.
+        should be a mix of the date period if the column an it's verbose name.
         :param date_period: a tuple of (start_date, end_date)
+        :param index: the index of the current field in the whole dates to be calculated
+        :param dates a list of tuples representing the start and the end date
         :return: a verbose string
         """
         dt_format = '%Y/%m/%d'
+
+        if pattern == 'monthly':
+            month_name = date_filter(date_period[0], 'F Y')
+            return f'{cls.verbose_name} {month_name}'
+        elif pattern == 'daily':
+            return f'{cls.verbose_name} {date_period[0].strftime(dt_format)}'
+        elif pattern == 'weekly':
+            return f' {cls.verbose_name} {_("Week")} {index} {date_period[0].strftime(dt_format)}'
+        elif pattern == 'yearly':
+            year = date_filter(date_period[0], 'Y')
+            return f'{cls.verbose_name} {year}'
+
         return f'{cls.verbose_name} {date_period[0].strftime(dt_format)} - {date_period[1].strftime(dt_format)}'
 
 
